use bevy::prelude::*;

use crate::{
    assets::AssetLoader,
    movement::{Heading, Mover, Speed, Velocity},
};

const PLAYER_SPEED: f32 = 100.;

#[derive(Component, Debug)]
pub struct Player;

impl Plugin for Player {
    fn build(&self, app: &mut App) {
        app.add_systems(PostStartup, spawn_player)
            .add_systems(Update, (sprite_select, controls.after(sprite_select)));
    }
}

fn spawn_player(mut commands: Commands, assets: Res<AssetLoader>) {
    let listener = SpatialListener::new(0.);

    commands
        .spawn((
            Player,
            SpriteSheetBundle {
                texture_atlas: assets.images.player.clone(),
                sprite: TextureAtlasSprite::new(0),
                ..Default::default()
            },
            Mover {
                velocity: Velocity(Vec2::ZERO),
                heading: Heading(Vec2::ZERO),
                speed: Speed(PLAYER_SPEED),
            },
        ))
        .with_children(|parent| {
            parent.spawn((SpatialBundle::default(), listener.clone()));
        });
}

fn controls(
    mut query: Query<(&mut TextureAtlasSprite, &mut Heading), With<Player>>,
    input: Res<Input<KeyCode>>,
) {
    let (mut sprite, mut heading) = query.single_mut();
    **heading = Vec2::ZERO;
    for key in input.get_pressed() {
        match key {
            KeyCode::Space => {}
            KeyCode::A | KeyCode::Left => {
                **heading -= Vec2::X;
            }
            KeyCode::D | KeyCode::Right => {
                **heading += Vec2::X;
            }
            KeyCode::W | KeyCode::Up => {
                **heading += Vec2::Y;
            }
            KeyCode::S | KeyCode::Down => {
                **heading -= Vec2::Y;
            }
            _ => {}
        }
    }
    if heading.y < 0. {
        sprite.index = 0;
    } else if heading.y > 0. {
        sprite.index = 1;
    } else if heading.x != 0. {
        sprite.index = 2;
        sprite.flip_x = heading.x > 0.;
    }
}

fn sprite_select(
    mut query: Query<(&Handle<TextureAtlas>, &mut TextureAtlasSprite), With<Player>>,
    keyboard_input: Res<Input<KeyCode>>,
    mut app_exit_events: ResMut<Events<bevy::app::AppExit>>,
    texture_atlases: Res<Assets<TextureAtlas>>,
) {
    if keyboard_input.pressed(KeyCode::Escape) {
        app_exit_events.send(bevy::app::AppExit);
    } else if keyboard_input.just_pressed(KeyCode::N) {
        let (texture, mut sprite) = query.single_mut();
        let t = texture_atlases
            .get(texture)
            .expect("could not load player texture");
        if sprite.index < t.textures.len() - 1 {
            sprite.index += 1;
        } else {
            sprite.index = 0;
        }
    }
}
